import sys
import os.path
import socket
import time
import datetime
import pytz
import math
try:
    from setproctitle import setproctitle
    setproctitle('sqmdaq_sdf')
except ImportError:
    pass

# Data acquisition routine for single stationary SQM-LE

def getdata(sqmcommand): # Procedure for sending rx, cx and ix requests
    if sqmcommand == 'rx':    strlen = 55
    elif sqmcommand == 'ix': strlen = 37
    elif sqmcommand == 'cx':    strlen = 56 # Expected lengths of SQM commands
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(10)
    s.connect((sqmurl,sqmport))
    s.send(sqmcommand)
    msg = ''
    while len(msg) < strlen: # Receives response from SQM
        chunk = s.recv(strlen-len(msg))
        if chunk == '':    raise RuntimeError, "Cannot connect to SQM!"
        msg = msg + chunk
    if sqmcommand == 'rx':    timenow = datetime.datetime.utcnow().replace(tzinfo=pytz.utc) # If rx sent and SQM responded, get current time
    s.close
    if sqmcommand == 'rx':    return msg, timenow
    else:    return msg # Returns SQM response and for rx command, also the current time in UTC

def datalinestr(): # Data line formatting
    utimestr = timenow.strftime("%Y-%m-%dT%H:%M:%S.")+(('{0:06d}').format(timenow.microsecond))[:3]+';'
    ltimestr = localtimenow.strftime("%Y-%m-%dT%H:%M:%S.")+(('{0:06d}').format(timenow.microsecond))[:3]+';'
    tempstr = str(int(rxstr[48:52]))+'.'+rxstr[53:54]+';'
    cyclestr = str(int(rxstr[23:33]))+';'
    freqstr = str(int(rxstr[10:20]))+';'
    magstr = str(int(rxstr[2:5]))+'.'+rxstr[6:8]
    return utimestr+ltimestr+tempstr+cyclestr+freqstr+magstr+'\r\n'

def newfileboolean(): # Determines if new file needs to be created
    inlogfile = open(outdir+lines[4]+'_lastdaq.log','r')
    try:
        lastfilestr = inlogfile.readline()
        lastdaqstr = inlogfile.readline()
        inlogfile.close()
        lastfile = datetime.datetime.strptime(lastfilestr[:19],"%Y-%m-%dT%H:%M:%S")+datetime.timedelta(microseconds=int(lastfilestr[20:23])*1000)
        lastfile = lastfile.replace(tzinfo=pytz.utc)
        lastdaq = datetime.datetime.strptime(lastdaqstr[:19],"%Y-%m-%dT%H:%M:%S")+datetime.timedelta(microseconds=int(lastfilestr[20:23])*1000)    
        lastdaq = lastdaq.replace(tzinfo=pytz.utc)
        if repint <= 300:
            if timenow-lastdaq > datetime.timedelta(minutes=5) or (lastfile < newfiletime and timenow > newfiletime):    return True
        elif repint >300:
            if timenow-lastdaq > 2 * datetime.timedelta(minutes=repint) or (lastfile < newfiletime and timenow > newfiletime):    return True
        else: return False
    except ValueError:    return True

def createfile(): # Procedure for creating new file, requires an existing .inp input file. Also creates a log file recording the time for last acquisition
    ixstr = getdata('ix')
    cxstr = getdata('cx')
    logfile = open(outdir+lines[4]+'_lastdaq.log','w')
    if lines[2][-1:] == 'u':    outfilename = timenow.strftime("%Y%m%d_%H%M%S")+'_'+lines[4]+'.dat'
    elif lines[2][-1:] == 'l':    outfilename = localtimenow.strftime("%Y%m%d_%H%M%S")+'_'+lines[4]+'.dat'
    outfile = open(outdir+outfilename,'w')
    outfile.write('# Community Standard Skyglow Data Format 1.0\r\n')
    outfile.write('# URL: http://www.darksky.org/NSBM/sdf1.0.pdf\r\n')
    outfile.write('# Number of header lines: 35\r\n')
    outfile.write('# This data is released under the following license: ODbL 1.0 http://opendatacommons.org/licenses/odbl/summary/\r\n')
    outfile.write('# Device type: '+lines[3]+'\r\n')
    outfile.write('# Instrument ID: '+lines[4]+'\r\n')
    outfile.write('# Data supplier: '+lines[5]+'\r\n')
    outfile.write('# Location name: '+lines[6]+'\r\n')
    outfile.write('# Position (lat, lon, elev(m)): '+lines[7]+'\r\n')
    outfile.write('# Local timezone: '+lines[8]+'\r\n')
    outfile.write('# Time Synchronization: '+lines[9]+'\r\n')
    outfile.write('# Moving / Stationary position: STATIONARY\r\n')
    outfile.write('# Moving / Fixed look direction: FIXED\r\n')
    outfile.write('# Number of channels: 1\r\n')
    outfile.write('# Filters per channel: '+lines[10]+'\r\n')
    outfile.write('# Measurement direction per channel: '+lines[11]+'\r\n')
    outfile.write('# Field of view (degrees): '+lines[12]+'\r\n')
    outfile.write('# Number of fields per line: 6\r\n')
    outfile.write('# SQM serial number: '+str(int(ixstr[29:37]))+'\r\n')
    outfile.write('# SQM firmware version: '+str(int(ixstr[2:10]))+'-'+str(int(ixstr[11:19]))+'-'+str(int(ixstr[20:28]))+'\r\n')
    outfile.write('# SQM cover offset value: '+lines[13]+'\r\n')
    outfile.write('# SQM readout test ix: '+ixstr+'\r\n')
    outfile.write('# SQM readout test rx: '+rxstr+'\r\n')
    outfile.write('# SQM readout test cx: '+cxstr+'\r\n')
    outfile.write('# Comment: '+lines[14]+'\r\n')
    outfile.write('# Comment: '+lines[15]+'\r\n')
    outfile.write('# Comment: '+lines[16]+'\r\n')
    outfile.write('# Comment: '+lines[17]+'\r\n')
    outfile.write('# Comment: '+lines[18]+'\r\n')
    outfile.write('# blank line 30\r\n')
    outfile.write('# blank line 31\r\n')
    outfile.write('# blank line 32\r\n')
    outfile.write('# UTC Date & Time, Local Date & Time, Temperature, Counts, Frequency, MSAS\r\n')
    outfile.write('# YYYY-MM-DDTHH:mm:ss.fff;YYYY-MM-DDTHH:mm:ss.fff;Celsius;number;Hz;mag/arcsec^2\r\n')
    outfile.write('# END OF HEADER\r\n')
    outfile.write(datalinestr())
    outfile.close()
    logfile.write(timenow.strftime("%Y-%m-%dT%H:%M:%S.")+(('{0:06d}').format(timenow.microsecond))[:3]+'\r\n')
    logfile.write(timenow.strftime("%Y-%m-%dT%H:%M:%S.")+(('{0:06d}').format(timenow.microsecond))[:3]+'\r\n')
    logfile.close()

def appendfile(): # Appends data to an existing data file
    logfile = open(outdir+lines[4]+'_lastdaq.log','r')
    lastfilestr = logfile.readline()
    lastfile = datetime.datetime.strptime(lastfilestr[:19],"%Y-%m-%dT%H:%M:%S")+datetime.timedelta(microseconds=int(lastfilestr[20:23])*1000)
    lastfile = lastfile.replace(tzinfo=pytz.utc)
    logfile.close # Reads log file and select latest file for appending
    logfile = open(outdir+lines[4]+'_lastdaq.log','w')
    logfile.write(lastfilestr)
    logfile.write(timenow.strftime("%Y-%m-%dT%H:%M:%S.")+(('{0:06d}').format(timenow.microsecond))[:3]+'\r\n')
    logfile.close() # Updates log file
    if lines[2][-1:] == 'l':    lastfile = lastfile.astimezone(pytz.timezone(lines[8]))
    outfilename = lastfile.strftime("%Y%m%d_%H%M%S")+'_'+lines[4]+'.dat'    
    outfile = open(outdir+outfilename,'a')
    outfile.write(datalinestr())
    outfile.close()


if len(sys.argv) < 2: # Help message when no arguments received
    print 'Usage:' + str(sys.argv[0]) + ' Parameter_file [Output_Directory]'
    print 'Data file will be saved in the directory of this program'
    print 'if no output location is specified'
elif len(sys.argv) == 2 and not os.path.isfile(sys.argv[1]):
    print 'Incorrect input parameter file. Nothing is written.'
else:
    if len(sys.argv) == 2: # Automatically uses current directory when destination unspecified
        outdir = './'
        print 'No output directory given. Using current directory'
    else:
        if (sys.argv[2])[-1:] != '/' or (sys.argv[2])[-1:] != '\\':
            outdir = (os.path.normcase(sys.argv[2])) + os.path.normcase('/')
        else:    outdir = os.path.normcase(sys.argv[2])
    paramfile = open(sys.argv[1], "r")
    lines = paramfile.readlines()
    for i in range(len(lines)):
        if lines[i][-2:] == '\r\n':
            lines[i] = lines[i][:-2]
        if lines[i][-1:] == '\n':
            lines[i] = lines[i][:-1]
        if lines[i][-1:] == '\r':
            lines[i] = lines[i][:-1]
    sqmurl = lines[0].split(':')[0]
    sqmport = int(lines[0].split(':')[1])
    n = int(lines[1].split(',')[0])
    if n == 0:    n -= 1
    repint = int(lines[1].split(',')[1])
    if not os.path.exists(outdir):    os.makedirs(outdir)
    while n != 0:
        timestart = (datetime.datetime.utcnow()).replace(microsecond=0)
        rxstr, timenow = getdata('rx')
        localtimenow = timenow.astimezone(pytz.timezone(lines[8]))
        if lines[2][-1:] == 'u':    newfiletime = timenow.replace(hour=int(lines[2][:2]),minute=int(lines[2][3:5]),second=0,microsecond=0)
        elif lines[2][-1:] == 'l':    newfiletime = localtimenow.replace(hour=int(lines[2][:2]),minute=int(lines[2][3:5]),second=0,microsecond=0)
        if not os.path.isfile(outdir+lines[4]+'_lastdaq.log'):    createfile()
        elif newfileboolean() == True:    createfile()
        else:    appendfile()
        if n > 0:
            n -= 1
            while (datetime.datetime.utcnow()).replace(microsecond=0) - timestart < datetime.timedelta(seconds=repint):
                time.sleep(0.001)
        if n < 0:
            while datetime.datetime.utcnow() - timestart < datetime.timedelta(seconds=repint):
                time.sleep(0.001)
